from __future__ import unicode_literals

import boto3

from moto import mock_secretsmanager
from botocore.exceptions import ClientError
import string
import unittest
import pytz
from datetime import datetime
from nose.tools import assert_raises
from six import b

DEFAULT_SECRET_NAME = 'test-secret'


@mock_secretsmanager
def test_get_secret_value():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    create_secret = conn.create_secret(Name='java-util-test-password',
                                       SecretString="foosecret")
    result = conn.get_secret_value(SecretId='java-util-test-password')
    assert result['SecretString'] == 'foosecret'

@mock_secretsmanager
def test_get_secret_value_binary():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    create_secret = conn.create_secret(Name='java-util-test-password',
                                       SecretBinary=b("foosecret"))
    result = conn.get_secret_value(SecretId='java-util-test-password')
    assert result['SecretBinary'] == b('foosecret')

@mock_secretsmanager
def test_get_secret_that_does_not_exist():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    with assert_raises(ClientError):
        result = conn.get_secret_value(SecretId='i-dont-exist')

@mock_secretsmanager
def test_get_secret_that_does_not_match():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    create_secret = conn.create_secret(Name='java-util-test-password',
                                       SecretString="foosecret")

    with assert_raises(ClientError):
        result = conn.get_secret_value(SecretId='i-dont-match')


@mock_secretsmanager
def test_get_secret_value_that_is_marked_deleted():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    conn.delete_secret(SecretId='test-secret')

    with assert_raises(ClientError):
        result = conn.get_secret_value(SecretId='test-secret')


@mock_secretsmanager
def test_create_secret():
    conn = boto3.client('secretsmanager', region_name='us-east-1')

    result = conn.create_secret(Name='test-secret', SecretString="foosecret")
    assert result['ARN']
    assert result['Name'] == 'test-secret'
    secret = conn.get_secret_value(SecretId='test-secret')
    assert secret['SecretString'] == 'foosecret'

@mock_secretsmanager
def test_create_secret_with_tags():
    conn = boto3.client('secretsmanager', region_name='us-east-1')
    secret_name = 'test-secret-with-tags'

    result = conn.create_secret(
        Name=secret_name,
        SecretString="foosecret",
        Tags=[{"Key": "Foo", "Value": "Bar"}, {"Key": "Mykey", "Value": "Myvalue"}]
    )
    assert result['ARN']
    assert result['Name'] == secret_name 
    secret_value = conn.get_secret_value(SecretId=secret_name)
    assert secret_value['SecretString'] == 'foosecret'
    secret_details = conn.describe_secret(SecretId=secret_name)
    assert secret_details['Tags'] == [{"Key": "Foo", "Value": "Bar"}, {"Key": "Mykey", "Value": "Myvalue"}]


@mock_secretsmanager
def test_delete_secret():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    deleted_secret = conn.delete_secret(SecretId='test-secret')

    assert deleted_secret['ARN']
    assert deleted_secret['Name'] == 'test-secret'
    assert deleted_secret['DeletionDate'] > datetime.fromtimestamp(1, pytz.utc)

    secret_details = conn.describe_secret(SecretId='test-secret')

    assert secret_details['ARN']
    assert secret_details['Name'] == 'test-secret'
    assert secret_details['DeletedDate'] > datetime.fromtimestamp(1, pytz.utc)


@mock_secretsmanager
def test_delete_secret_force():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    result = conn.delete_secret(SecretId='test-secret', ForceDeleteWithoutRecovery=True)

    assert result['ARN']
    assert result['DeletionDate'] > datetime.fromtimestamp(1, pytz.utc)
    assert result['Name'] == 'test-secret'

    with assert_raises(ClientError):
        result = conn.get_secret_value(SecretId='test-secret')


@mock_secretsmanager
def test_delete_secret_that_does_not_exist():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    with assert_raises(ClientError):
        result = conn.delete_secret(SecretId='i-dont-exist', ForceDeleteWithoutRecovery=True)


@mock_secretsmanager
def test_delete_secret_fails_with_both_force_delete_flag_and_recovery_window_flag():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    with assert_raises(ClientError):
        result = conn.delete_secret(SecretId='test-secret', RecoveryWindowInDays=1, ForceDeleteWithoutRecovery=True)


@mock_secretsmanager
def test_delete_secret_recovery_window_too_short():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    with assert_raises(ClientError):
        result = conn.delete_secret(SecretId='test-secret', RecoveryWindowInDays=6)


@mock_secretsmanager
def test_delete_secret_recovery_window_too_long():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    with assert_raises(ClientError):
        result = conn.delete_secret(SecretId='test-secret', RecoveryWindowInDays=31)


@mock_secretsmanager
def test_delete_secret_that_is_marked_deleted():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    deleted_secret = conn.delete_secret(SecretId='test-secret')

    with assert_raises(ClientError):
        result = conn.delete_secret(SecretId='test-secret')


@mock_secretsmanager
def test_get_random_password_default_length():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password()
    assert len(random_password['RandomPassword']) == 32

@mock_secretsmanager
def test_get_random_password_default_requirements():
    # When require_each_included_type, default true
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password()
    # Should contain lowercase, upppercase, digit, special character
    assert any(c.islower() for c in random_password['RandomPassword'])
    assert any(c.isupper() for c in random_password['RandomPassword'])
    assert any(c.isdigit() for c in random_password['RandomPassword'])
    assert any(c in string.punctuation
               for c in random_password['RandomPassword'])

@mock_secretsmanager
def test_get_random_password_custom_length():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password(PasswordLength=50)
    assert len(random_password['RandomPassword']) == 50

@mock_secretsmanager
def test_get_random_exclude_lowercase():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password(PasswordLength=55,
                                               ExcludeLowercase=True)
    assert any(c.islower() for c in random_password['RandomPassword']) == False

@mock_secretsmanager
def test_get_random_exclude_uppercase():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password(PasswordLength=55,
                                               ExcludeUppercase=True)
    assert any(c.isupper() for c in random_password['RandomPassword']) == False

@mock_secretsmanager
def test_get_random_exclude_characters_and_symbols():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password(PasswordLength=20,
                                               ExcludeCharacters='xyzDje@?!.')
    assert any(c in 'xyzDje@?!.' for c in random_password['RandomPassword']) == False

@mock_secretsmanager
def test_get_random_exclude_numbers():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password(PasswordLength=100,
                                               ExcludeNumbers=True)
    assert any(c.isdigit() for c in random_password['RandomPassword']) == False

@mock_secretsmanager
def test_get_random_exclude_punctuation():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password(PasswordLength=100,
                                               ExcludePunctuation=True)
    assert any(c in string.punctuation
               for c in random_password['RandomPassword']) == False

@mock_secretsmanager
def test_get_random_include_space_false():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password(PasswordLength=300)
    assert any(c.isspace() for c in random_password['RandomPassword']) == False

@mock_secretsmanager
def test_get_random_include_space_true():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password(PasswordLength=4,
                                               IncludeSpace=True)
    assert any(c.isspace() for c in random_password['RandomPassword']) == True

@mock_secretsmanager
def test_get_random_require_each_included_type():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    random_password = conn.get_random_password(PasswordLength=4,
                                               RequireEachIncludedType=True)
    assert any(c in string.punctuation for c in random_password['RandomPassword']) == True
    assert any(c in string.ascii_lowercase for c in random_password['RandomPassword']) == True
    assert any(c in string.ascii_uppercase for c in random_password['RandomPassword']) == True
    assert any(c in string.digits for c in random_password['RandomPassword']) == True

@mock_secretsmanager
def test_get_random_too_short_password():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    with assert_raises(ClientError):
        random_password = conn.get_random_password(PasswordLength=3)

@mock_secretsmanager
def test_get_random_too_long_password():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    with assert_raises(Exception):
        random_password = conn.get_random_password(PasswordLength=5555)

@mock_secretsmanager
def test_describe_secret():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')
    
    conn.create_secret(Name='test-secret-2',
                       SecretString='barsecret')
    
    secret_description = conn.describe_secret(SecretId='test-secret')
    secret_description_2 = conn.describe_secret(SecretId='test-secret-2')

    assert secret_description   # Returned dict is not empty
    assert secret_description['Name'] == ('test-secret')
    assert secret_description['ARN'] != '' # Test arn not empty
    assert secret_description_2['Name'] == ('test-secret-2')
    assert secret_description_2['ARN'] != '' # Test arn not empty

@mock_secretsmanager
def test_describe_secret_that_does_not_exist():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    with assert_raises(ClientError):
        result = conn.get_secret_value(SecretId='i-dont-exist')

@mock_secretsmanager
def test_describe_secret_that_does_not_match():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')
    
    with assert_raises(ClientError):
        result = conn.get_secret_value(SecretId='i-dont-match')


@mock_secretsmanager
def test_list_secrets_empty():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    secrets = conn.list_secrets()

    assert secrets['SecretList'] == []


@mock_secretsmanager
def test_list_secrets():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    conn.create_secret(Name='test-secret-2',
                       SecretString='barsecret',
                       Tags=[{
                           'Key': 'a',
                           'Value': '1'
                       }])

    secrets = conn.list_secrets()

    assert secrets['SecretList'][0]['ARN'] is not None
    assert secrets['SecretList'][0]['Name'] == 'test-secret'
    assert secrets['SecretList'][1]['ARN'] is not None
    assert secrets['SecretList'][1]['Name'] == 'test-secret-2'
    assert secrets['SecretList'][1]['Tags'] == [{
        'Key': 'a',
        'Value': '1'
    }]


@mock_secretsmanager
def test_restore_secret():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    conn.delete_secret(SecretId='test-secret')

    described_secret_before = conn.describe_secret(SecretId='test-secret')
    assert described_secret_before['DeletedDate'] > datetime.fromtimestamp(1, pytz.utc)

    restored_secret = conn.restore_secret(SecretId='test-secret')
    assert restored_secret['ARN']
    assert restored_secret['Name'] == 'test-secret'

    described_secret_after = conn.describe_secret(SecretId='test-secret')
    assert 'DeletedDate' not in described_secret_after


@mock_secretsmanager
def test_restore_secret_that_is_not_deleted():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    restored_secret = conn.restore_secret(SecretId='test-secret')
    assert restored_secret['ARN']
    assert restored_secret['Name'] == 'test-secret'


@mock_secretsmanager
def test_restore_secret_that_does_not_exist():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    with assert_raises(ClientError):
        result = conn.restore_secret(SecretId='i-dont-exist')


@mock_secretsmanager
def test_rotate_secret():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    conn.create_secret(Name=DEFAULT_SECRET_NAME,
                       SecretString='foosecret')

    rotated_secret = conn.rotate_secret(SecretId=DEFAULT_SECRET_NAME)

    assert rotated_secret
    assert rotated_secret['ARN'] != '' # Test arn not empty
    assert rotated_secret['Name'] == DEFAULT_SECRET_NAME
    assert rotated_secret['VersionId'] != ''

@mock_secretsmanager
def test_rotate_secret_enable_rotation():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    conn.create_secret(Name=DEFAULT_SECRET_NAME,
                       SecretString='foosecret')

    initial_description = conn.describe_secret(SecretId=DEFAULT_SECRET_NAME)
    assert initial_description
    assert initial_description['RotationEnabled'] is False
    assert initial_description['RotationRules']['AutomaticallyAfterDays'] == 0

    conn.rotate_secret(SecretId=DEFAULT_SECRET_NAME,
                       RotationRules={'AutomaticallyAfterDays': 42})

    rotated_description = conn.describe_secret(SecretId=DEFAULT_SECRET_NAME)
    assert rotated_description
    assert rotated_description['RotationEnabled'] is True
    assert rotated_description['RotationRules']['AutomaticallyAfterDays'] == 42


@mock_secretsmanager
def test_rotate_secret_that_is_marked_deleted():
    conn = boto3.client('secretsmanager', region_name='us-west-2')

    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    conn.delete_secret(SecretId='test-secret')

    with assert_raises(ClientError):
        result = conn.rotate_secret(SecretId='test-secret')


@mock_secretsmanager
def test_rotate_secret_that_does_not_exist():
    conn = boto3.client('secretsmanager', 'us-west-2')

    with assert_raises(ClientError):
        result = conn.rotate_secret(SecretId='i-dont-exist')

@mock_secretsmanager
def test_rotate_secret_that_does_not_match():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    conn.create_secret(Name='test-secret',
                       SecretString='foosecret')

    with assert_raises(ClientError):
        result = conn.rotate_secret(SecretId='i-dont-match')

@mock_secretsmanager
def test_rotate_secret_client_request_token_too_short():
    # Test is intentionally empty. Boto3 catches too short ClientRequestToken
    # and raises ParamValidationError before Moto can see it.
    # test_server actually handles this error.
    assert True

@mock_secretsmanager
def test_rotate_secret_client_request_token_too_long():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    conn.create_secret(Name=DEFAULT_SECRET_NAME,
                       SecretString='foosecret')

    client_request_token = (
        'ED9F8B6C-85B7-446A-B7E4-38F2A3BEB13C-'
        'ED9F8B6C-85B7-446A-B7E4-38F2A3BEB13C'
    )
    with assert_raises(ClientError):
        result = conn.rotate_secret(SecretId=DEFAULT_SECRET_NAME,
                                    ClientRequestToken=client_request_token)

@mock_secretsmanager
def test_rotate_secret_rotation_lambda_arn_too_long():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    conn.create_secret(Name=DEFAULT_SECRET_NAME,
                       SecretString='foosecret')

    rotation_lambda_arn = '85B7-446A-B7E4' * 147    # == 2058 characters
    with assert_raises(ClientError):
        result = conn.rotate_secret(SecretId=DEFAULT_SECRET_NAME,
                                    RotationLambdaARN=rotation_lambda_arn)

@mock_secretsmanager
def test_rotate_secret_rotation_period_zero():
    # Test is intentionally empty. Boto3 catches zero day rotation period
    # and raises ParamValidationError before Moto can see it.
    # test_server actually handles this error.
    assert True

@mock_secretsmanager
def test_rotate_secret_rotation_period_too_long():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    conn.create_secret(Name=DEFAULT_SECRET_NAME,
                       SecretString='foosecret')

    rotation_rules = {'AutomaticallyAfterDays': 1001}
    with assert_raises(ClientError):
        result = conn.rotate_secret(SecretId=DEFAULT_SECRET_NAME,
                                    RotationRules=rotation_rules)

@mock_secretsmanager
def test_put_secret_value_puts_new_secret():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    put_secret_value_dict = conn.put_secret_value(SecretId=DEFAULT_SECRET_NAME,
                                                  SecretString='foosecret',
                                                  VersionStages=['AWSCURRENT'])
    version_id = put_secret_value_dict['VersionId']

    get_secret_value_dict = conn.get_secret_value(SecretId=DEFAULT_SECRET_NAME,
                                                  VersionId=version_id,
                                                  VersionStage='AWSCURRENT')

    assert get_secret_value_dict
    assert get_secret_value_dict['SecretString'] == 'foosecret'

@mock_secretsmanager
def test_put_secret_value_can_get_first_version_if_put_twice():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    put_secret_value_dict = conn.put_secret_value(SecretId=DEFAULT_SECRET_NAME,
                                                  SecretString='first_secret',
                                                  VersionStages=['AWSCURRENT'])
    first_version_id = put_secret_value_dict['VersionId']
    conn.put_secret_value(SecretId=DEFAULT_SECRET_NAME,
                          SecretString='second_secret',
                          VersionStages=['AWSCURRENT'])

    first_secret_value_dict = conn.get_secret_value(SecretId=DEFAULT_SECRET_NAME,
                                                    VersionId=first_version_id)
    first_secret_value = first_secret_value_dict['SecretString']

    assert first_secret_value == 'first_secret'


@mock_secretsmanager
def test_put_secret_value_versions_differ_if_same_secret_put_twice():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    put_secret_value_dict = conn.put_secret_value(SecretId=DEFAULT_SECRET_NAME,
                                                  SecretString='dupe_secret',
                                                  VersionStages=['AWSCURRENT'])
    first_version_id = put_secret_value_dict['VersionId']
    put_secret_value_dict = conn.put_secret_value(SecretId=DEFAULT_SECRET_NAME,
                                                  SecretString='dupe_secret',
                                                  VersionStages=['AWSCURRENT'])
    second_version_id = put_secret_value_dict['VersionId']

    assert first_version_id != second_version_id


@mock_secretsmanager
def test_can_list_secret_version_ids():
    conn = boto3.client('secretsmanager', region_name='us-west-2')
    put_secret_value_dict = conn.put_secret_value(SecretId=DEFAULT_SECRET_NAME,
                                                  SecretString='dupe_secret',
                                                  VersionStages=['AWSCURRENT'])
    first_version_id = put_secret_value_dict['VersionId']
    put_secret_value_dict = conn.put_secret_value(SecretId=DEFAULT_SECRET_NAME,
                                                  SecretString='dupe_secret',
                                                  VersionStages=['AWSCURRENT'])
    second_version_id = put_secret_value_dict['VersionId']

    versions_list = conn.list_secret_version_ids(SecretId=DEFAULT_SECRET_NAME)

    returned_version_ids = [v['VersionId'] for v in versions_list['Versions']]

    assert [first_version_id, second_version_id].sort() == returned_version_ids.sort()




from __future__ import unicode_literals
from collections import defaultdict
import datetime
import json

from moto.compat import OrderedDict
from moto.core import BaseBackend, BaseModel
from moto.core.utils import unix_time
from .comparisons import get_comparison_func


class DynamoJsonEncoder(json.JSONEncoder):

    def default(self, obj):
        if hasattr(obj, 'to_json'):
            return obj.to_json()


def dynamo_json_dump(dynamo_object):
    return json.dumps(dynamo_object, cls=DynamoJsonEncoder)


class DynamoType(object):
    """
    http://docs.aws.amazon.com/amazondynamodb/latest/developerguide/DataModel.html#DataModelDataTypes
    """

    def __init__(self, type_as_dict):
        self.type = list(type_as_dict.keys())[0]
        self.value = list(type_as_dict.values())[0]

    def __hash__(self):
        return hash((self.type, self.value))

    def __eq__(self, other):
        return (
            self.type == other.type and
            self.value == other.value
        )

    def __repr__(self):
        return "DynamoType: {0}".format(self.to_json())

    def to_json(self):
        return {self.type: self.value}

    def compare(self, range_comparison, range_objs):
        """
        Compares this type against comparison filters
        """
        range_values = [obj.value for obj in range_objs]
        comparison_func = get_comparison_func(range_comparison)
        return comparison_func(self.value, *range_values)


class Item(BaseModel):

    def __init__(self, hash_key, hash_key_type, range_key, range_key_type, attrs):
        self.hash_key = hash_key
        self.hash_key_type = hash_key_type
        self.range_key = range_key
        self.range_key_type = range_key_type

        self.attrs = {}
        for key, value in attrs.items():
            self.attrs[key] = DynamoType(value)

    def __repr__(self):
        return "Item: {0}".format(self.to_json())

    def to_json(self):
        attributes = {}
        for attribute_key, attribute in self.attrs.items():
            attributes[attribute_key] = attribute.value

        return {
            "Attributes": attributes
        }

    def describe_attrs(self, attributes):
        if attributes:
            included = {}
            for key, value in self.attrs.items():
                if key in attributes:
                    included[key] = value
        else:
            included = self.attrs
        return {
            "Item": included
        }


class Table(BaseModel):

    def __init__(self, name, hash_key_attr, hash_key_type,
                 range_key_attr=None, range_key_type=None, read_capacity=None,
                 write_capacity=None):
        self.name = name
        self.hash_key_attr = hash_key_attr
        self.hash_key_type = hash_key_type
        self.range_key_attr = range_key_attr
        self.range_key_type = range_key_type
        self.read_capacity = read_capacity
        self.write_capacity = write_capacity
        self.created_at = datetime.datetime.utcnow()
        self.items = defaultdict(dict)

    @property
    def has_range_key(self):
        return self.range_key_attr is not None

    @property
    def describe(self):
        results = {
            "Table": {
                "CreationDateTime": unix_time(self.created_at),
                "KeySchema": {
                    "HashKeyElement": {
                        "AttributeName": self.hash_key_attr,
                        "AttributeType": self.hash_key_type
                    },
                },
                "ProvisionedThroughput": {
                    "ReadCapacityUnits": self.read_capacity,
                    "WriteCapacityUnits": self.write_capacity
                },
                "TableName": self.name,
                "TableStatus": "ACTIVE",
                "ItemCount": len(self),
                "TableSizeBytes": 0,
            }
        }
        if self.has_range_key:
            results["Table"]["KeySchema"]["RangeKeyElement"] = {
                "AttributeName": self.range_key_attr,
                "AttributeType": self.range_key_type
            }
        return results

    @classmethod
    def create_from_cloudformation_json(cls, resource_name, cloudformation_json, region_name):
        properties = cloudformation_json['Properties']
        key_attr = [i['AttributeName'] for i in properties['KeySchema'] if i['KeyType'] == 'HASH'][0]
        key_type = [i['AttributeType'] for i in properties['AttributeDefinitions'] if i['AttributeName'] == key_attr][0]
        spec = {
            'name': properties['TableName'],
            'hash_key_attr': key_attr,
            'hash_key_type': key_type
        }
        # TODO: optional properties still missing:
        # range_key_attr, range_key_type, read_capacity, write_capacity
        return Table(**spec)

    def __len__(self):
        count = 0
        for key, value in self.items.items():
            if self.has_range_key:
                count += len(value)
            else:
                count += 1
        return count

    def __nonzero__(self):
        return True

    def __bool__(self):
        return self.__nonzero__()

    def put_item(self, item_attrs):
        hash_value = DynamoType(item_attrs.get(self.hash_key_attr))
        if self.has_range_key:
            range_value = DynamoType(item_attrs.get(self.range_key_attr))
        else:
            range_value = None

        item = Item(hash_value, self.hash_key_type, range_value,
                    self.range_key_type, item_attrs)

        if range_value:
            self.items[hash_value][range_value] = item
        else:
            self.items[hash_value] = item
        return item

    def get_item(self, hash_key, range_key):
        if self.has_range_key and not range_key:
            raise ValueError(
                "Table has a range key, but no range key was passed into get_item")
        try:
            if range_key:
                return self.items[hash_key][range_key]
            else:
                return self.items[hash_key]
        except KeyError:
            return None

    def query(self, hash_key, range_comparison, range_objs):
        results = []
        last_page = True  # Once pagination is implemented, change this

        if self.range_key_attr:
            possible_results = self.items[hash_key].values()
        else:
            possible_results = list(self.all_items())

        if range_comparison:
            for result in possible_results:
                if result.range_key.compare(range_comparison, range_objs):
                    results.append(result)
        else:
            # If we're not filtering on range key, return all values
            results = possible_results
        return results, last_page

    def all_items(self):
        for hash_set in self.items.values():
            if self.range_key_attr:
                for item in hash_set.values():
                    yield item
            else:
                yield hash_set

    def scan(self, filters):
        results = []
        scanned_count = 0
        last_page = True  # Once pagination is implemented, change this

        for result in self.all_items():
            scanned_count += 1
            passes_all_conditions = True
            for attribute_name, (comparison_operator, comparison_objs) in filters.items():
                attribute = result.attrs.get(attribute_name)

                if attribute:
                    # Attribute found
                    if not attribute.compare(comparison_operator, comparison_objs):
                        passes_all_conditions = False
                        break
                elif comparison_operator == 'NULL':
                    # Comparison is NULL and we don't have the attribute
                    continue
                else:
                    # No attribute found and comparison is no NULL. This item
                    # fails
                    passes_all_conditions = False
                    break

            if passes_all_conditions:
                results.append(result)

        return results, scanned_count, last_page

    def delete_item(self, hash_key, range_key):
        try:
            if range_key:
                return self.items[hash_key].pop(range_key)
            else:
                return self.items.pop(hash_key)
        except KeyError:
            return None

    def get_cfn_attribute(self, attribute_name):
        from moto.cloudformation.exceptions import UnformattedGetAttTemplateException
        if attribute_name == 'StreamArn':
            region = 'us-east-1'
            time = '2000-01-01T00:00:00.000'
            return 'arn:aws:dynamodb:{0}:123456789012:table/{1}/stream/{2}'.format(region, self.name, time)
        raise UnformattedGetAttTemplateException()


class DynamoDBBackend(BaseBackend):

    def __init__(self):
        self.tables = OrderedDict()

    def create_table(self, name, **params):
        table = Table(name, **params)
        self.tables[name] = table
        return table

    def delete_table(self, name):
        return self.tables.pop(name, None)

    def update_table_throughput(self, name, new_read_units, new_write_units):
        table = self.tables[name]
        table.read_capacity = new_read_units
        table.write_capacity = new_write_units
        return table

    def put_item(self, table_name, item_attrs):
        table = self.tables.get(table_name)
        if not table:
            return None

        return table.put_item(item_attrs)

    def get_item(self, table_name, hash_key_dict, range_key_dict):
        table = self.tables.get(table_name)
        if not table:
            return None

        hash_key = DynamoType(hash_key_dict)
        range_key = DynamoType(range_key_dict) if range_key_dict else None

        return table.get_item(hash_key, range_key)

    def query(self, table_name, hash_key_dict, range_comparison, range_value_dicts):
        table = self.tables.get(table_name)
        if not table:
            return None, None

        hash_key = DynamoType(hash_key_dict)
        range_values = [DynamoType(range_value)
                        for range_value in range_value_dicts]

        return table.query(hash_key, range_comparison, range_values)

    def scan(self, table_name, filters):
        table = self.tables.get(table_name)
        if not table:
            return None, None, None

        scan_filters = {}
        for key, (comparison_operator, comparison_values) in filters.items():
            dynamo_types = [DynamoType(value) for value in comparison_values]
            scan_filters[key] = (comparison_operator, dynamo_types)

        return table.scan(scan_filters)

    def delete_item(self, table_name, hash_key_dict, range_key_dict):
        table = self.tables.get(table_name)
        if not table:
            return None

        hash_key = DynamoType(hash_key_dict)
        range_key = DynamoType(range_key_dict) if range_key_dict else None

        return table.delete_item(hash_key, range_key)


dynamodb_backend = DynamoDBBackend()



from __future__ import unicode_literals
from moto.core.exceptions import RESTError


class ELBClientError(RESTError):
    code = 400


class DuplicateTagKeysError(ELBClientError):

    def __init__(self, cidr):
        super(DuplicateTagKeysError, self).__init__(
            "DuplicateTagKeys",
            "Tag key was specified more than once: {0}"
            .format(cidr))


class LoadBalancerNotFoundError(ELBClientError):

    def __init__(self, cidr):
        super(LoadBalancerNotFoundError, self).__init__(
            "LoadBalancerNotFound",
            "The specified load balancer does not exist: {0}"
            .format(cidr))


class TooManyTagsError(ELBClientError):

    def __init__(self):
        super(TooManyTagsError, self).__init__(
            "LoadBalancerNotFound",
            "The quota for the number of tags that can be assigned to a load balancer has been reached")


class BadHealthCheckDefinition(ELBClientError):

    def __init__(self):
        super(BadHealthCheckDefinition, self).__init__(
            "ValidationError",
            "HealthCheck Target must begin with one of HTTP, TCP, HTTPS, SSL")


class DuplicateListenerError(ELBClientError):

    def __init__(self, name, port):
        super(DuplicateListenerError, self).__init__(
            "DuplicateListener",
            "A listener already exists for {0} with LoadBalancerPort {1}, but with a different InstancePort, Protocol, or SSLCertificateId"
            .format(name, port))


class DuplicateLoadBalancerName(ELBClientError):

    def __init__(self, name):
        super(DuplicateLoadBalancerName, self).__init__(
            "DuplicateLoadBalancerName",
            "The specified load balancer name already exists for this account: {0}"
            .format(name))


class EmptyListenersError(ELBClientError):

    def __init__(self):
        super(EmptyListenersError, self).__init__(
            "ValidationError",
            "Listeners cannot be empty")


class InvalidSecurityGroupError(ELBClientError):

    def __init__(self):
        super(InvalidSecurityGroupError, self).__init__(
            "ValidationError",
            "One or more of the specified security groups do not exist.")



from __future__ import unicode_literals

import random
import string
import six
import re


def random_password(password_length, exclude_characters, exclude_numbers,
                    exclude_punctuation, exclude_uppercase, exclude_lowercase,
                    include_space, require_each_included_type):

    password = ''
    required_characters = ''

    if not exclude_lowercase and not exclude_uppercase:
        password += string.ascii_letters
        required_characters += random.choice(_exclude_characters(
            string.ascii_lowercase, exclude_characters))
        required_characters += random.choice(_exclude_characters(
            string.ascii_uppercase, exclude_characters))
    elif not exclude_lowercase:
        password += string.ascii_lowercase
        required_characters += random.choice(_exclude_characters(
            string.ascii_lowercase, exclude_characters))
    elif not exclude_uppercase:
        password += string.ascii_uppercase
        required_characters += random.choice(_exclude_characters(
            string.ascii_uppercase, exclude_characters))
    if not exclude_numbers:
        password += string.digits
        required_characters += random.choice(_exclude_characters(
            string.digits, exclude_characters))
    if not exclude_punctuation:
        password += string.punctuation
        required_characters += random.choice(_exclude_characters(
            string.punctuation, exclude_characters))
    if include_space:
        password += " "
        required_characters += " "

    password = ''.join(
        six.text_type(random.choice(password))
        for x in range(password_length))

    if require_each_included_type:
        password = _add_password_require_each_included_type(
            password, required_characters)

    password = _exclude_characters(password, exclude_characters)
    return password


def secret_arn(region, secret_id):
    id_string = ''.join(random.choice(string.ascii_letters) for _ in range(5))
    return "arn:aws:secretsmanager:{0}:1234567890:secret:{1}-{2}".format(
        region, secret_id, id_string)


def _exclude_characters(password, exclude_characters):
    for c in exclude_characters:
        if c in string.punctuation:
            # Escape punctuation regex usage
            c = "\{0}".format(c)
        password = re.sub(c, '', str(password))
    return password


def _add_password_require_each_included_type(password, required_characters):
    password_with_required_char = password[:-len(required_characters)]
    password_with_required_char += required_characters

    return password_with_required_char



from __future__ import unicode_literals
from .responses import SQSResponse

url_bases = [
    "https?://(.*?)(queue|sqs)(.*?).amazonaws.com"
]

dispatch = SQSResponse().dispatch

url_paths = {
    '{0}/$': dispatch,
    '{0}/(?P<account_id>\d+)/(?P<queue_name>[a-zA-Z0-9\-_\.]+)': dispatch,
}



from __future__ import unicode_literals
import json

try:
    from urllib import unquote
except ImportError:
    from urllib.parse import unquote

from moto.core.responses import BaseResponse
from .models import resourcegroups_backends


class ResourceGroupsResponse(BaseResponse):
    SERVICE_NAME = 'resource-groups'

    @property
    def resourcegroups_backend(self):
        return resourcegroups_backends[self.region]

    def create_group(self):
        name = self._get_param("Name")
        description = self._get_param("Description")
        resource_query = self._get_param("ResourceQuery")
        tags = self._get_param("Tags")
        group = self.resourcegroups_backend.create_group(
            name=name,
            description=description,
            resource_query=resource_query,
            tags=tags,
        )
        return json.dumps({
            "Group": {
                "GroupArn": group.arn,
                "Name": group.name,
                "Description": group.description
            },
            "ResourceQuery": group.resource_query,
            "Tags": group.tags
        })

    def delete_group(self):
        group_name = self._get_param("GroupName")
        group = self.resourcegroups_backend.delete_group(group_name=group_name)
        return json.dumps({
            "Group": {
                "GroupArn": group.arn,
                "Name": group.name,
                "Description": group.description
            },
        })

    def get_group(self):
        group_name = self._get_param("GroupName")
        group = self.resourcegroups_backend.get_group(group_name=group_name)
        return json.dumps({
            "Group": {
                "GroupArn": group.arn,
                "Name": group.name,
                "Description": group.description,
            }
        })

    def get_group_query(self):
        group_name = self._get_param("GroupName")
        group = self.resourcegroups_backend.get_group(group_name=group_name)
        return json.dumps({
            "GroupQuery": {
                "GroupName": group.name,
                "ResourceQuery": group.resource_query,
            }
        })

    def get_tags(self):
        arn = unquote(self._get_param("Arn"))
        return json.dumps({
            "Arn": arn,
            "Tags": self.resourcegroups_backend.get_tags(arn=arn)
        })

    def list_group_resources(self):
        raise NotImplementedError('ResourceGroups.list_group_resources is not yet implemented')

    def list_groups(self):
        filters = self._get_param("Filters")
        if filters:
            raise NotImplementedError(
                'ResourceGroups.list_groups with filter parameter is not yet implemented'
            )
        max_results = self._get_int_param("MaxResults", 50)
        next_token = self._get_param("NextToken")
        groups = self.resourcegroups_backend.list_groups(
            filters=filters,
            max_results=max_results,
            next_token=next_token
        )
        return json.dumps({
            "GroupIdentifiers": [{
                "GroupName": group.name,
                "GroupArn": group.arn,
            } for group in groups.values()],
            "Groups": [{
                "GroupArn": group.arn,
                "Name": group.name,
                "Description": group.description,
            } for group in groups.values()],
            "NextToken": next_token,
        })

    def search_resources(self):
        raise NotImplementedError('ResourceGroups.search_resources is not yet implemented')

    def tag(self):
        arn = unquote(self._get_param("Arn"))
        tags = self._get_param("Tags")
        if arn not in self.resourcegroups_backend.groups.by_arn:
            raise NotImplementedError(
                'ResourceGroups.tag with non-resource-group Arn parameter is not yet implemented'
            )
        self.resourcegroups_backend.tag(arn=arn, tags=tags)
        return json.dumps({
            "Arn": arn,
            "Tags": tags
        })

    def untag(self):
        arn = unquote(self._get_param("Arn"))
        keys = self._get_param("Keys")
        if arn not in self.resourcegroups_backend.groups.by_arn:
            raise NotImplementedError(
                'ResourceGroups.untag with non-resource-group Arn parameter is not yet implemented'
            )
        self.resourcegroups_backend.untag(arn=arn, keys=keys)
        return json.dumps({
            "Arn": arn,
            "Keys": keys
        })

    def update_group(self):
        group_name = self._get_param("GroupName")
        description = self._get_param("Description", "")
        group = self.resourcegroups_backend.update_group(group_name=group_name, description=description)
        return json.dumps({
            "Group": {
                "GroupArn": group.arn,
                "Name": group.name,
                "Description": group.description
            },
        })

    def update_group_query(self):
        group_name = self._get_param("GroupName")
        resource_query = self._get_param("ResourceQuery")
        group = self.resourcegroups_backend.update_group_query(
            group_name=group_name,
            resource_query=resource_query
        )
        return json.dumps({
            "GroupQuery": {
                "GroupName": group.name,
                "ResourceQuery": resource_query
            }
        })



from __future__ import unicode_literals
from moto.core.exceptions import JsonRESTError


class SecretsManagerClientError(JsonRESTError):
    code = 400


class ResourceNotFoundException(SecretsManagerClientError):
    def __init__(self):
        self.code = 404
        super(ResourceNotFoundException, self).__init__(
            "ResourceNotFoundException",
            "Secrets Manager can't find the specified secret"
        )


class ClientError(SecretsManagerClientError):
    def __init__(self, message):
        super(ClientError, self).__init__(
            'InvalidParameterValue',
            message)


class InvalidParameterException(SecretsManagerClientError):
    def __init__(self, message):
        super(InvalidParameterException, self).__init__(
            'InvalidParameterException',
            message)


class ResourceExistsException(SecretsManagerClientError):
    def __init__(self, message):
        super(ResourceExistsException, self).__init__(
            'ResourceExistsException',
            message
        )


class InvalidRequestException(SecretsManagerClientError):
    def __init__(self, message):
        super(InvalidRequestException, self).__init__(
            'InvalidRequestException',
            message)



from __future__ import unicode_literals
import json
import six
import re

from moto.core.responses import BaseResponse
from moto.core.utils import camelcase_to_underscores, amzn_request_id
from .exceptions import InvalidIndexNameError
from .models import dynamodb_backends, dynamo_json_dump


def has_empty_keys_or_values(_dict):
    if _dict == "":
        return True
    if not isinstance(_dict, dict):
        return False
    return any(
        key == '' or value == '' or
        has_empty_keys_or_values(value)
        for key, value in _dict.items()
    )


def get_empty_str_error():
    er = 'com.amazonaws.dynamodb.v20111205#ValidationException'
    return (400,
            {'server': 'amazon.com'},
            dynamo_json_dump({'__type': er,
                              'message': ('One or more parameter values were '
                                          'invalid: An AttributeValue may not '
                                          'contain an empty string')}
                             ))


class DynamoHandler(BaseResponse):

    def get_endpoint_name(self, headers):
        """Parses request headers and extracts part od the X-Amz-Target
        that corresponds to a method of DynamoHandler

        ie: X-Amz-Target: DynamoDB_20111205.ListTables -> ListTables
        """
        # Headers are case-insensitive. Probably a better way to do this.
        match = headers.get('x-amz-target') or headers.get('X-Amz-Target')
        if match:
            return match.split(".")[1]

    def error(self, type_, message, status=400):
        return status, self.response_headers, dynamo_json_dump({'__type': type_, 'message': message})

    @property
    def dynamodb_backend(self):
        """
        :return: DynamoDB2 Backend
        :rtype: moto.dynamodb2.models.DynamoDBBackend
        """
        return dynamodb_backends[self.region]

    @amzn_request_id
    def call_action(self):
        self.body = json.loads(self.body or '{}')
        endpoint = self.get_endpoint_name(self.headers)
        if endpoint:
            endpoint = camelcase_to_underscores(endpoint)
            response = getattr(self, endpoint)()
            if isinstance(response, six.string_types):
                return 200, self.response_headers, response

            else:
                status_code, new_headers, response_content = response
                self.response_headers.update(new_headers)
                return status_code, self.response_headers, response_content
        else:
            return 404, self.response_headers, ""

    def list_tables(self):
        body = self.body
        limit = body.get('Limit', 100)
        if body.get("ExclusiveStartTableName"):
            last = body.get("ExclusiveStartTableName")
            start = list(self.dynamodb_backend.tables.keys()).index(last) + 1
        else:
            start = 0
        all_tables = list(self.dynamodb_backend.tables.keys())
        if limit:
            tables = all_tables[start:start + limit]
        else:
            tables = all_tables[start:]
        response = {"TableNames": tables}
        if limit and len(all_tables) > start + limit:
            response["LastEvaluatedTableName"] = tables[-1]

        return dynamo_json_dump(response)

    def create_table(self):
        body = self.body
        # get the table name
        table_name = body['TableName']
        # check billing mode and get the throughput
        if "BillingMode" in body.keys() and body["BillingMode"] == "PAY_PER_REQUEST":
            if "ProvisionedThroughput" in body.keys():
                er = 'com.amazonaws.dynamodb.v20111205#ValidationException'
                return self.error(er,
                                  'ProvisionedThroughput cannot be specified \
                                   when BillingMode is PAY_PER_REQUEST')
            throughput = None
        else:         # Provisioned (default billing mode)
            throughput = body.get("ProvisionedThroughput")
        # getting the schema
        key_schema = body['KeySchema']
        # getting attribute definition
        attr = body["AttributeDefinitions"]
        # getting the indexes
        global_indexes = body.get("GlobalSecondaryIndexes", [])
        local_secondary_indexes = body.get("LocalSecondaryIndexes", [])
        # get the stream specification
        streams = body.get("StreamSpecification")

        table = self.dynamodb_backend.create_table(table_name,
                                                   schema=key_schema,
                                                   throughput=throughput,
                                                   attr=attr,
                                                   global_indexes=global_indexes,
                                                   indexes=local_secondary_indexes,
                                                   streams=streams)
        if table is not None:
            return dynamo_json_dump(table.describe())
        else:
            er = 'com.amazonaws.dynamodb.v20111205#ResourceInUseException'
            return self.error(er, 'Resource in use')

    def delete_table(self):
        name = self.body['TableName']
        table = self.dynamodb_backend.delete_table(name)
        if table is not None:
            return dynamo_json_dump(table.describe())
        else:
            er = 'com.amazonaws.dynamodb.v20111205#ResourceNotFoundException'
            return self.error(er, 'Requested resource not found')

    def tag_resource(self):
        table_arn = self.body['ResourceArn']
        tags = self.body['Tags']
        self.dynamodb_backend.tag_resource(table_arn, tags)
        return ''

    def untag_resource(self):
        table_arn = self.body['ResourceArn']
        tags = self.body['TagKeys']
        self.dynamodb_backend.untag_resource(table_arn, tags)
        return ''

    def list_tags_of_resource(self):
        try:
            table_arn = self.body['ResourceArn']
            all_tags = self.dynamodb_backend.list_tags_of_resource(table_arn)
            all_tag_keys = [tag['Key'] for tag in all_tags]
            marker = self.body.get('NextToken')
            if marker:
                start = all_tag_keys.index(marker) + 1
            else:
                start = 0
            max_items = 10  # there is no default, but using 10 to make testing easier
            tags_resp = all_tags[start:start + max_items]
            next_marker = None
            if len(all_tags) > start + max_items:
                next_marker = tags_resp[-1]['Key']
            if next_marker:
                return json.dumps({'Tags': tags_resp,
                                   'NextToken': next_marker})
            return json.dumps({'Tags': tags_resp})
        except AttributeError:
            er = 'com.amazonaws.dynamodb.v20111205#ResourceNotFoundException'
            return self.error(er, 'Requested resource not found')

    def update_table(self):
        name = self.body['TableName']
        table = self.dynamodb_backend.get_table(name)
        if 'GlobalSecondaryIndexUpdates' in self.body:
            table = self.dynamodb_backend.update_table_global_indexes(
                name, self.body['GlobalSecondaryIndexUpdates'])
        if 'ProvisionedThroughput' in self.body:
            throughput = self.body["ProvisionedThroughput"]
            table = self.dynamodb_backend.update_table_throughput(name, throughput)
        if 'StreamSpecification' in self.body:
            try:
                table = self.dynamodb_backend.update_table_streams(name, self.body['StreamSpecification'])
            except ValueError:
                er = 'com.amazonaws.dynamodb.v20111205#ResourceInUseException'
                return self.error(er, 'Cannot enable stream')

        return dynamo_json_dump(table.describe())

    def describe_table(self):
        name = self.body['TableName']
        try:
            table = self.dynamodb_backend.tables[name]
        except KeyError:
            er = 'com.amazonaws.dynamodb.v20111205#ResourceNotFoundException'
            return self.error(er, 'Requested resource not found')
        return dynamo_json_dump(table.describe(base_key='Table'))

    def put_item(self):
        name = self.body['TableName']
        item = self.body['Item']
        return_values = self.body.get('ReturnValues', 'NONE')

        if return_values not in ('ALL_OLD', 'NONE'):
            er = 'com.amazonaws.dynamodb.v20111205#ValidationException'
            return self.error(er, 'Return values set to invalid value')

        if has_empty_keys_or_values(item):
            return get_empty_str_error()

        overwrite = 'Expected' not in self.body
        if not overwrite:
            expected = self.body['Expected']
        else:
            expected = None

        if return_values == 'ALL_OLD':
            existing_item = self.dynamodb_backend.get_item(name, item)
            if existing_item:
                existing_attributes = existing_item.to_json()['Attributes']
            else:
                existing_attributes = {}

        # Attempt to parse simple ConditionExpressions into an Expected
        # expression
        condition_expression = self.body.get('ConditionExpression')
        expression_attribute_names = self.body.get('ExpressionAttributeNames', {})
        expression_attribute_values = self.body.get('ExpressionAttributeValues', {})

        if condition_expression:
            overwrite = False

        try:
            result = self.dynamodb_backend.put_item(
                name, item, expected, condition_expression,
                expression_attribute_names, expression_attribute_values,
                overwrite)
        except ValueError:
            er = 'com.amazonaws.dynamodb.v20111205#ConditionalCheckFailedException'
            return self.error(er, 'A condition specified in the operation could not be evaluated.')

        if result:
            item_dict = result.to_json()
            item_dict['ConsumedCapacity'] = {
                'TableName': name,
                'CapacityUnits': 1
            }
            if return_values == 'ALL_OLD':
                item_dict['Attributes'] = existing_attributes
            else:
                item_dict.pop('Attributes', None)
            return dynamo_json_dump(item_dict)
        else:
            er = 'com.amazonaws.dynamodb.v20111205#ResourceNotFoundException'
            return self.error(er, 'Requested resource not found')

    def batch_write_item(self):
        table_batches = self.body['RequestItems']

        for table_name, table_requests in table_batches.items():
            for table_request in table_requests:
                request_type = list(table_request.keys())[0]
                request = list(table_request.values())[0]
                if request_type == 'PutRequest':
                    item = request['Item']
                    self.dynamodb_backend.put_item(table_name, item)
                elif request_type == 'DeleteRequest':
                    keys = request['Key']
                    item = self.dynamodb_backend.delete_item(table_name, keys)

        response = {
            "ConsumedCapacity": [
                {
                    'TableName': table_name,
                    'CapacityUnits': 1.0,
                    'Table': {'CapacityUnits': 1.0}
                } for table_name, table_requests in table_batches.items()
            ],
            "ItemCollectionMetrics": {},
            "UnprocessedItems": {}
        }

        return dynamo_json_dump(response)

    def get_item(self):
        name = self.body['TableName']
        key = self.body['Key']
        try:
            item = self.dynamodb_backend.get_item(name, key)
        except ValueError:
            er = 'com.amazon.coral.validate#ValidationException'
            return self.error(er, 'Validation Exception')
        if item:
            item_dict = item.describe_attrs(attributes=None)
            item_dict['ConsumedCapacity'] = {
                'TableName': name,
                'CapacityUnits': 0.5
            }
            return dynamo_json_dump(item_dict)
        else:
            # Item not found
            return 200, self.response_headers, '{}'

    def batch_get_item(self):
        table_batches = self.body['RequestItems']

        results = {
            "ConsumedCapacity": [],
            "Responses": {
            },
            "UnprocessedKeys": {
            }
        }

        for table_name, table_request in table_batches.items():
            keys = table_request['Keys']
            attributes_to_get = table_request.get('AttributesToGet')
            results["Responses"][table_name] = []
            for key in keys:
                item = self.dynamodb_backend.get_item(table_name, key)
                if item:
                    item_describe = item.describe_attrs(attributes_to_get)
                    results["Responses"][table_name].append(
                        item_describe["Item"])

            results["ConsumedCapacity"].append({
                "CapacityUnits": len(keys),
                "TableName": table_name
            })
        return dynamo_json_dump(results)

    def query(self):
        name = self.body['TableName']
        # {u'KeyConditionExpression': u'#n0 = :v0', u'ExpressionAttributeValues': {u':v0': {u'S': u'johndoe'}}, u'ExpressionAttributeNames': {u'#n0': u'username'}}
        key_condition_expression = self.body.get('KeyConditionExpression')
        projection_expression = self.body.get('ProjectionExpression')
        expression_attribute_names = self.body.get('ExpressionAttributeNames', {})
        filter_expression = self.body.get('FilterExpression')
        expression_attribute_values = self.body.get('ExpressionAttributeValues', {})

        if projection_expression and expression_attribute_names:
            expressions = [x.strip() for x in projection_expression.split(',')]
            for expression in expressions:
                if expression in expression_attribute_names:
                    projection_expression = projection_expression.replace(expression, expression_attribute_names[expression])

        filter_kwargs = {}

        if key_condition_expression:
            value_alias_map = self.body.get('ExpressionAttributeValues', {})

            table = self.dynamodb_backend.get_table(name)

            # If table does not exist
            if table is None:
                return self.error('com.amazonaws.dynamodb.v20120810#ResourceNotFoundException',
                                  'Requested resource not found')

            index_name = self.body.get('IndexName')
            if index_name:
                all_indexes = (table.global_indexes or []) + \
                    (table.indexes or [])
                indexes_by_name = dict((i['IndexName'], i)
                                       for i in all_indexes)
                if index_name not in indexes_by_name:
                    raise ValueError('Invalid index: %s for table: %s. Available indexes are: %s' % (
                        index_name, name, ', '.join(indexes_by_name.keys())
                    ))

                index = indexes_by_name[index_name]['KeySchema']
            else:
                index = table.schema

            reverse_attribute_lookup = dict((v, k) for k, v in
                                            six.iteritems(self.body.get('ExpressionAttributeNames', {})))

            if " AND " in key_condition_expression:
                expressions = key_condition_expression.split(" AND ", 1)

                index_hash_key = [key for key in index if key['KeyType'] == 'HASH'][0]
                hash_key_var = reverse_attribute_lookup.get(index_hash_key['AttributeName'],
                                                            index_hash_key['AttributeName'])
                hash_key_regex = r'(^|[\s(]){0}\b'.format(hash_key_var)
                i, hash_key_expression = next((i, e) for i, e in enumerate(expressions)
                                              if re.search(hash_key_regex, e))
                hash_key_expression = hash_key_expression.strip('()')
                expressions.pop(i)

                # TODO implement more than one range expression and OR operators
                range_key_expression = expressions[0].strip('()')
                range_key_expression_components = range_key_expression.split()
                range_comparison = range_key_expression_components[1]

                if 'AND' in range_key_expression:
                    range_comparison = 'BETWEEN'
                    range_values = [
                        value_alias_map[range_key_expression_components[2]],
                        value_alias_map[range_key_expression_components[4]],
                    ]
                elif 'begins_with' in range_key_expression:
                    range_comparison = 'BEGINS_WITH'
                    range_values = [
                        value_alias_map[range_key_expression_components[1]],
                    ]
                else:
                    range_values = [value_alias_map[
                        range_key_expression_components[2]]]
            else:
                hash_key_expression = key_condition_expression.strip('()')
                range_comparison = None
                range_values = []

            hash_key_value_alias = hash_key_expression.split("=")[1].strip()
            # Temporary fix until we get proper KeyConditionExpression function
            hash_key = value_alias_map.get(hash_key_value_alias, {'S': hash_key_value_alias})
        else:
            # 'KeyConditions': {u'forum_name': {u'ComparisonOperator': u'EQ', u'AttributeValueList': [{u'S': u'the-key'}]}}
            key_conditions = self.body.get('KeyConditions')
            query_filters = self.body.get("QueryFilter")
            if key_conditions:
                hash_key_name, range_key_name = self.dynamodb_backend.get_table_keys_name(
                    name, key_conditions.keys())
                for key, value in key_conditions.items():
                    if key not in (hash_key_name, range_key_name):
                        filter_kwargs[key] = value
                if hash_key_name is None:
                    er = "'com.amazonaws.dynamodb.v20120810#ResourceNotFoundException"
                    return self.error(er, 'Requested resource not found')
                hash_key = key_conditions[hash_key_name][
                    'AttributeValueList'][0]
                if len(key_conditions) == 1:
                    range_comparison = None
                    range_values = []
                else:
                    if range_key_name is None and not filter_kwargs:
                        er = "com.amazon.coral.validate#ValidationException"
                        return self.error(er, 'Validation Exception')
                    else:
                        range_condition = key_conditions.get(range_key_name)
                        if range_condition:
                            range_comparison = range_condition[
                                'ComparisonOperator']
                            range_values = range_condition[
                                'AttributeValueList']
                        else:
                            range_comparison = None
                            range_values = []
            if query_filters:
                filter_kwargs.update(query_filters)
        index_name = self.body.get('IndexName')
        exclusive_start_key = self.body.get('ExclusiveStartKey')
        limit = self.body.get("Limit")
        scan_index_forward = self.body.get("ScanIndexForward")
        items, scanned_count, last_evaluated_key = self.dynamodb_backend.query(
            name, hash_key, range_comparison, range_values, limit,
            exclusive_start_key, scan_index_forward, projection_expression, index_name=index_name,
            expr_names=expression_attribute_names, expr_values=expression_attribute_values,
            filter_expression=filter_expression, **filter_kwargs
        )
        if items is None:
            er = 'com.amazonaws.dynamodb.v20111205#ResourceNotFoundException'
            return self.error(er, 'Requested resource not found')

        result = {
            "Count": len(items),
            'ConsumedCapacity': {
                'TableName': name,
                'CapacityUnits': 1,
            },
            "ScannedCount": scanned_count
        }

        if self.body.get('Select', '').upper() != 'COUNT':
            result["Items"] = [item.attrs for item in items]

        if last_evaluated_key is not None:
            result["LastEvaluatedKey"] = last_evaluated_key

        return dynamo_json_dump(result)

    def scan(self):
        name = self.body['TableName']

        filters = {}
        scan_filters = self.body.get('ScanFilter', {})
        for attribute_name, scan_filter in scan_filters.items():
            # Keys are attribute names. Values are tuples of (comparison,
            # comparison_value)
            comparison_operator = scan_filter["ComparisonOperator"]
            comparison_values = scan_filter.get("AttributeValueList", [])
            filters[attribute_name] = (comparison_operator, comparison_values)

        filter_expression = self.body.get('FilterExpression')
        expression_attribute_values = self.body.get('ExpressionAttributeValues', {})
        expression_attribute_names = self.body.get('ExpressionAttributeNames', {})
        projection_expression = self.body.get('ProjectionExpression', '')
        exclusive_start_key = self.body.get('ExclusiveStartKey')
        limit = self.body.get("Limit")
        index_name = self.body.get('IndexName')

        try:
            items, scanned_count, last_evaluated_key = self.dynamodb_backend.scan(name, filters,
                                                                                  limit,
                                                                                  exclusive_start_key,
                                                                                  filter_expression,
                                                                                  expression_attribute_names,
                                                                                  expression_attribute_values,
                                                                                  index_name,
                                                                                  projection_expression)
        except InvalidIndexNameError as err:
            er = 'com.amazonaws.dynamodb.v20111205#ValidationException'
            return self.error(er, str(err))
        except ValueError as err:
            er = 'com.amazonaws.dynamodb.v20111205#ValidationError'
            return self.error(er, 'Bad Filter Expression: {0}'.format(err))
        except Exception as err:
            er = 'com.amazonaws.dynamodb.v20111205#InternalFailure'
            return self.error(er, 'Internal error. {0}'.format(err))

        # Items should be a list, at least an empty one. Is None if table does not exist.
        # Should really check this at the beginning
        if items is None:
            er = 'com.amazonaws.dynamodb.v20111205#ResourceNotFoundException'
            return self.error(er, 'Requested resource not found')

        result = {
            "Count": len(items),
            "Items": [item.attrs for item in items],
            'ConsumedCapacity': {
                'TableName': name,
                'CapacityUnits': 1,
            },
            "ScannedCount": scanned_count
        }
        if last_evaluated_key is not None:
            result["LastEvaluatedKey"] = last_evaluated_key
        return dynamo_json_dump(result)

    def delete_item(self):
        name = self.body['TableName']
        keys = self.body['Key']
        return_values = self.body.get('ReturnValues', 'NONE')
        if return_values not in ('ALL_OLD', 'NONE'):
            er = 'com.amazonaws.dynamodb.v20111205#ValidationException'
            return self.error(er, 'Return values set to invalid value')

        table = self.dynamodb_backend.get_table(name)
        if not table:
            er = 'com.amazonaws.dynamodb.v20120810#ConditionalCheckFailedException'
            return self.error(er, 'A condition specified in the operation could not be evaluated.')

        item = self.dynamodb_backend.delete_item(name, keys)
        if item and return_values == 'ALL_OLD':
            item_dict = item.to_json()
        else:
            item_dict = {'Attributes': {}}
        item_dict['ConsumedCapacityUnits'] = 0.5
        return dynamo_json_dump(item_dict)

    def update_item(self):
        name = self.body['TableName']
        key = self.body['Key']
        return_values = self.body.get('ReturnValues', 'NONE')
        update_expression = self.body.get('UpdateExpression', '').strip()
        attribute_updates = self.body.get('AttributeUpdates')
        expression_attribute_names = self.body.get(
            'ExpressionAttributeNames', {})
        expression_attribute_values = self.body.get(
            'ExpressionAttributeValues', {})
        existing_item = self.dynamodb_backend.get_item(name, key)
        if existing_item:
            existing_attributes = existing_item.to_json()['Attributes']
        else:
            existing_attributes = {}

        if return_values not in ('NONE', 'ALL_OLD', 'ALL_NEW', 'UPDATED_OLD',
                                 'UPDATED_NEW'):
            er = 'com.amazonaws.dynamodb.v20111205#ValidationException'
            return self.error(er, 'Return values set to invalid value')

        if has_empty_keys_or_values(expression_attribute_values):
            return get_empty_str_error()

        if 'Expected' in self.body:
            expected = self.body['Expected']
        else:
            expected = None

        # Attempt to parse simple ConditionExpressions into an Expected
        # expression
        condition_expression = self.body.get('ConditionExpression')
        expression_attribute_names = self.body.get('ExpressionAttributeNames', {})
        expression_attribute_values = self.body.get('ExpressionAttributeValues', {})

        # Support spaces between operators in an update expression
        # E.g. `a = b + c` -> `a=b+c`
        if update_expression:
            update_expression = re.sub(
                '\s*([=\+-])\s*', '\\1', update_expression)

        try:
            item = self.dynamodb_backend.update_item(
                name, key, update_expression, attribute_updates, expression_attribute_names,
                expression_attribute_values, expected, condition_expression
            )
        except ValueError:
            er = 'com.amazonaws.dynamodb.v20111205#ConditionalCheckFailedException'
            return self.error(er, 'A condition specified in the operation could not be evaluated.')
        except TypeError:
            er = 'com.amazonaws.dynamodb.v20111205#ValidationException'
            return self.error(er, 'Validation Exception')

        item_dict = item.to_json()
        item_dict['ConsumedCapacity'] = {
            'TableName': name,
            'CapacityUnits': 0.5
        }
        unchanged_attributes = {
            k for k in existing_attributes.keys()
            if existing_attributes[k] == item_dict['Attributes'].get(k)
        }
        changed_attributes = set(existing_attributes.keys()).union(item_dict['Attributes'].keys()).difference(unchanged_attributes)

        if return_values == 'NONE':
            item_dict['Attributes'] = {}
        elif return_values == 'ALL_OLD':
            item_dict['Attributes'] = existing_attributes
        elif return_values == 'UPDATED_OLD':
            item_dict['Attributes'] = {
                k: v for k, v in existing_attributes.items()
                if k in changed_attributes
            }
        elif return_values == 'UPDATED_NEW':
            item_dict['Attributes'] = {
                k: v for k, v in item_dict['Attributes'].items()
                if k in changed_attributes
            }

        return dynamo_json_dump(item_dict)

    def describe_limits(self):
        return json.dumps({
            'AccountMaxReadCapacityUnits': 20000,
            'TableMaxWriteCapacityUnits': 10000,
            'AccountMaxWriteCapacityUnits': 20000,
            'TableMaxReadCapacityUnits': 10000
        })

    def update_time_to_live(self):
        name = self.body['TableName']
        ttl_spec = self.body['TimeToLiveSpecification']

        self.dynamodb_backend.update_ttl(name, ttl_spec)

        return json.dumps({'TimeToLiveSpecification': ttl_spec})

    def describe_time_to_live(self):
        name = self.body['TableName']

        ttl_spec = self.dynamodb_backend.describe_ttl(name)

        return json.dumps({'TimeToLiveDescription': ttl_spec})



from __future__ import unicode_literals
from moto.core.exceptions import JsonRESTError


class IoTClientError(JsonRESTError):
    code = 400


class ResourceNotFoundException(IoTClientError):
    def __init__(self):
        self.code = 404
        super(ResourceNotFoundException, self).__init__(
            "ResourceNotFoundException",
            "The specified resource does not exist"
        )


class InvalidRequestException(IoTClientError):
    def __init__(self, msg=None):
        self.code = 400
        super(InvalidRequestException, self).__init__(
            "InvalidRequestException",
            msg or "The request is not valid."
        )


class VersionConflictException(IoTClientError):
    def __init__(self, name):
        self.code = 409
        super(VersionConflictException, self).__init__(
            'VersionConflictException',
            'The version for thing %s does not match the expected version.' % name
        )


class CertificateStateException(IoTClientError):
    def __init__(self, msg, cert_id):
        self.code = 406
        super(CertificateStateException, self).__init__(
            'CertificateStateException',
            '%s Id: %s' % (msg, cert_id)
        )


class DeleteConflictException(IoTClientError):
    def __init__(self, msg):
        self.code = 409
        super(DeleteConflictException, self).__init__(
            'DeleteConflictException', msg
        )



from __future__ import unicode_literals
from .responses import EmailResponse

url_bases = [
    "https?://email.(.+).amazonaws.com",
    "https?://ses.(.+).amazonaws.com",
]

url_paths = {
    '{0}/$': EmailResponse.dispatch,
}



import random
import boto3
import json

from moto.events import mock_events
from botocore.exceptions import ClientError
from nose.tools import assert_raises

RULES = [
    {'Name': 'test1', 'ScheduleExpression': 'rate(5 minutes)'},
    {'Name': 'test2', 'ScheduleExpression': 'rate(1 minute)'},
    {'Name': 'test3', 'EventPattern': '{"source": ["test-source"]}'}
]

TARGETS = {
    'test-target-1': {
        'Id': 'test-target-1',
        'Arn': 'arn:aws:lambda:us-west-2:111111111111:function:test-function-1',
        'Rules': ['test1', 'test2']
    },
    'test-target-2': {
        'Id': 'test-target-2',
        'Arn': 'arn:aws:lambda:us-west-2:111111111111:function:test-function-2',
        'Rules': ['test1', 'test3']
    },
    'test-target-3': {
        'Id': 'test-target-3',
        'Arn': 'arn:aws:lambda:us-west-2:111111111111:function:test-function-3',
        'Rules': ['test1', 'test2']
    },
    'test-target-4': {
        'Id': 'test-target-4',
        'Arn': 'arn:aws:lambda:us-west-2:111111111111:function:test-function-4',
        'Rules': ['test1', 'test3']
    },
    'test-target-5': {
        'Id': 'test-target-5',
        'Arn': 'arn:aws:lambda:us-west-2:111111111111:function:test-function-5',
        'Rules': ['test1', 'test2']
    },
    'test-target-6': {
        'Id': 'test-target-6',
        'Arn': 'arn:aws:lambda:us-west-2:111111111111:function:test-function-6',
        'Rules': ['test1', 'test3']
    }
}


def get_random_rule():
    return RULES[random.randint(0, len(RULES) - 1)]


def generate_environment():
    client = boto3.client('events', 'us-west-2')

    for rule in RULES:
        client.put_rule(
            Name=rule['Name'],
            ScheduleExpression=rule.get('ScheduleExpression', ''),
            EventPattern=rule.get('EventPattern', '')
        )

        targets = []
        for target in TARGETS:
            if rule['Name'] in TARGETS[target].get('Rules'):
                targets.append({'Id': target, 'Arn': TARGETS[target]['Arn']})

        client.put_targets(Rule=rule['Name'], Targets=targets)

    return client


@mock_events
def test_list_rules():
    client = generate_environment()
    response = client.list_rules()

    assert(response is not None)
    assert(len(response['Rules']) > 0)


@mock_events
def test_describe_rule():
    rule_name = get_random_rule()['Name']
    client = generate_environment()
    response = client.describe_rule(Name=rule_name)

    assert(response is not None)
    assert(response.get('Name') == rule_name)
    assert(response.get('Arn') is not None)


@mock_events
def test_enable_disable_rule():
    rule_name = get_random_rule()['Name']
    client = generate_environment()

    # Rules should start out enabled in these tests.
    rule = client.describe_rule(Name=rule_name)
    assert(rule['State'] == 'ENABLED')

    client.disable_rule(Name=rule_name)
    rule = client.describe_rule(Name=rule_name)
    assert(rule['State'] == 'DISABLED')

    client.enable_rule(Name=rule_name)
    rule = client.describe_rule(Name=rule_name)
    assert(rule['State'] == 'ENABLED')

    # Test invalid name
    try:
        client.enable_rule(Name='junk')

    except ClientError as ce:
        assert ce.response['Error']['Code'] == 'ResourceNotFoundException'


@mock_events
def test_list_rule_names_by_target():
    test_1_target = TARGETS['test-target-1']
    test_2_target = TARGETS['test-target-2']
    client = generate_environment()

    rules = client.list_rule_names_by_target(TargetArn=test_1_target['Arn'])
    assert(len(rules['RuleNames']) == len(test_1_target['Rules']))
    for rule in rules['RuleNames']:
        assert(rule in test_1_target['Rules'])

    rules = client.list_rule_names_by_target(TargetArn=test_2_target['Arn'])
    assert(len(rules['RuleNames']) == len(test_2_target['Rules']))
    for rule in rules['RuleNames']:
        assert(rule in test_2_target['Rules'])


@mock_events
def test_list_rules():
    client = generate_environment()

    rules = client.list_rules()
    assert(len(rules['Rules']) == len(RULES))


@mock_events
def test_delete_rule():
    client = generate_environment()

    client.delete_rule(Name=RULES[0]['Name'])
    rules = client.list_rules()
    assert(len(rules['Rules']) == len(RULES) - 1)


@mock_events
def test_list_targets_by_rule():
    rule_name = get_random_rule()['Name']
    client = generate_environment()
    targets = client.list_targets_by_rule(Rule=rule_name)

    expected_targets = []
    for target in TARGETS:
        if rule_name in TARGETS[target].get('Rules'):
            expected_targets.append(target)

    assert(len(targets['Targets']) == len(expected_targets))


@mock_events
def test_remove_targets():
    rule_name = get_random_rule()['Name']
    client = generate_environment()

    targets = client.list_targets_by_rule(Rule=rule_name)['Targets']
    targets_before = len(targets)
    assert(targets_before > 0)

    client.remove_targets(Rule=rule_name, Ids=[targets[0]['Id']])

    targets = client.list_targets_by_rule(Rule=rule_name)['Targets']
    targets_after = len(targets)
    assert(targets_before - 1 == targets_after)


@mock_events
def test_permissions():
    client = boto3.client('events', 'eu-central-1')

    client.put_permission(Action='events:PutEvents', Principal='111111111111', StatementId='Account1')
    client.put_permission(Action='events:PutEvents', Principal='222222222222', StatementId='Account2')

    resp = client.describe_event_bus()
    resp_policy = json.loads(resp['Policy'])
    assert len(resp_policy['Statement']) == 2

    client.remove_permission(StatementId='Account2')

    resp = client.describe_event_bus()
    resp_policy = json.loads(resp['Policy'])
    assert len(resp_policy['Statement']) == 1
    assert resp_policy['Statement'][0]['Sid'] == 'Account1'


@mock_events
def test_put_events():
    client = boto3.client('events', 'eu-central-1')

    event = {
        "Source": "com.mycompany.myapp",
        "Detail": '{"key1": "value3", "key2": "value4"}',
        "Resources": ["resource1", "resource2"],
        "DetailType": "myDetailType"
    }

    client.put_events(Entries=[event])
    # Boto3 would error if it didn't return 200 OK

    with assert_raises(ClientError):
        client.put_events(Entries=[event]*20)



from __future__ import unicode_literals

import boto.datapipeline
import sure  # noqa

from moto import mock_datapipeline_deprecated
from moto.datapipeline.utils import remove_capitalization_of_dict_keys


def get_value_from_fields(key, fields):
    for field in fields:
        if field['key'] == key:
            return field['stringValue']


@mock_datapipeline_deprecated
def test_create_pipeline():
    conn = boto.datapipeline.connect_to_region("us-west-2")

    res = conn.create_pipeline("mypipeline", "some-unique-id")

    pipeline_id = res["pipelineId"]
    pipeline_descriptions = conn.describe_pipelines(
        [pipeline_id])["pipelineDescriptionList"]
    pipeline_descriptions.should.have.length_of(1)

    pipeline_description = pipeline_descriptions[0]
    pipeline_description['name'].should.equal("mypipeline")
    pipeline_description["pipelineId"].should.equal(pipeline_id)
    fields = pipeline_description['fields']

    get_value_from_fields('@pipelineState', fields).should.equal("PENDING")
    get_value_from_fields('uniqueId', fields).should.equal("some-unique-id")


PIPELINE_OBJECTS = [
    {
        "id": "Default",
        "name": "Default",
        "fields": [{
            "key": "workerGroup",
            "stringValue": "workerGroup"
        }]
    },
    {
        "id": "Schedule",
        "name": "Schedule",
        "fields": [{
            "key": "startDateTime",
            "stringValue": "2012-12-12T00:00:00"
        }, {
            "key": "type",
            "stringValue": "Schedule"
        }, {
            "key": "period",
            "stringValue": "1 hour"
        }, {
            "key": "endDateTime",
            "stringValue": "2012-12-21T18:00:00"
        }]
    },
    {
        "id": "SayHello",
        "name": "SayHello",
        "fields": [{
            "key": "type",
            "stringValue": "ShellCommandActivity"
        }, {
            "key": "command",
            "stringValue": "echo hello"
        }, {
            "key": "parent",
            "refValue": "Default"
        }, {
            "key": "schedule",
            "refValue": "Schedule"
        }]
    }
]


@mock_datapipeline_deprecated
def test_creating_pipeline_definition():
    conn = boto.datapipeline.connect_to_region("us-west-2")
    res = conn.create_pipeline("mypipeline", "some-unique-id")
    pipeline_id = res["pipelineId"]

    conn.put_pipeline_definition(PIPELINE_OBJECTS, pipeline_id)

    pipeline_definition = conn.get_pipeline_definition(pipeline_id)
    pipeline_definition['pipelineObjects'].should.have.length_of(3)
    default_object = pipeline_definition['pipelineObjects'][0]
    default_object['name'].should.equal("Default")
    default_object['id'].should.equal("Default")
    default_object['fields'].should.equal([{
        "key": "workerGroup",
        "stringValue": "workerGroup"
    }])


@mock_datapipeline_deprecated
def test_describing_pipeline_objects():
    conn = boto.datapipeline.connect_to_region("us-west-2")
    res = conn.create_pipeline("mypipeline", "some-unique-id")
    pipeline_id = res["pipelineId"]

    conn.put_pipeline_definition(PIPELINE_OBJECTS, pipeline_id)

    objects = conn.describe_objects(["Schedule", "Default"], pipeline_id)[
        'pipelineObjects']

    objects.should.have.length_of(2)
    default_object = [x for x in objects if x['id'] == 'Default'][0]
    default_object['name'].should.equal("Default")
    default_object['fields'].should.equal([{
        "key": "workerGroup",
        "stringValue": "workerGroup"
    }])


@mock_datapipeline_deprecated
def test_activate_pipeline():
    conn = boto.datapipeline.connect_to_region("us-west-2")

    res = conn.create_pipeline("mypipeline", "some-unique-id")

    pipeline_id = res["pipelineId"]
    conn.activate_pipeline(pipeline_id)

    pipeline_descriptions = conn.describe_pipelines(
        [pipeline_id])["pipelineDescriptionList"]
    pipeline_descriptions.should.have.length_of(1)
    pipeline_description = pipeline_descriptions[0]
    fields = pipeline_description['fields']

    get_value_from_fields('@pipelineState', fields).should.equal("SCHEDULED")


@mock_datapipeline_deprecated
def test_delete_pipeline():
    conn = boto.datapipeline.connect_to_region("us-west-2")
    res = conn.create_pipeline("mypipeline", "some-unique-id")
    pipeline_id = res["pipelineId"]

    conn.delete_pipeline(pipeline_id)

    response = conn.list_pipelines()

    response["pipelineIdList"].should.have.length_of(0)


@mock_datapipeline_deprecated
def test_listing_pipelines():
    conn = boto.datapipeline.connect_to_region("us-west-2")
    res1 = conn.create_pipeline("mypipeline1", "some-unique-id1")
    res2 = conn.create_pipeline("mypipeline2", "some-unique-id2")

    response = conn.list_pipelines()

    response["hasMoreResults"].should.be(False)
    response["marker"].should.be.none
    response["pipelineIdList"].should.have.length_of(2)
    response["pipelineIdList"].should.contain({
        "id": res1["pipelineId"],
        "name": "mypipeline1",
    })
    response["pipelineIdList"].should.contain({
        "id": res2["pipelineId"],
        "name": "mypipeline2"
    })


@mock_datapipeline_deprecated
def test_listing_paginated_pipelines():
    conn = boto.datapipeline.connect_to_region("us-west-2")
    for i in range(100):
        conn.create_pipeline("mypipeline%d" % i, "some-unique-id%d" % i)

    response = conn.list_pipelines()

    response["hasMoreResults"].should.be(True)
    response["marker"].should.equal(response["pipelineIdList"][-1]['id'])
    response["pipelineIdList"].should.have.length_of(50)


# testing a helper function
def test_remove_capitalization_of_dict_keys():
    result = remove_capitalization_of_dict_keys(
        {
            "Id": "IdValue",
            "Fields": [{
                "Key": "KeyValue",
                "StringValue": "StringValueValue"
            }]
        }
    )

    result.should.equal({
        "id": "IdValue",
        "fields": [{
            "key": "KeyValue",
            "stringValue": "StringValueValue"
        }],
    })



from __future__ import unicode_literals

TABLE_INPUT = {
    'Owner': 'a_fake_owner',
    'Parameters': {
        'EXTERNAL': 'TRUE',
    },
    'Retention': 0,
    'StorageDescriptor': {
        'BucketColumns': [],
        'Compressed': False,
        'InputFormat': 'org.apache.hadoop.hive.ql.io.parquet.MapredParquetInputFormat',
        'NumberOfBuckets': -1,
        'OutputFormat': 'org.apache.hadoop.hive.ql.io.parquet.MapredParquetOutputFormat',
        'Parameters': {},
        'SerdeInfo': {
            'Parameters': {
                'serialization.format': '1'
            },
            'SerializationLibrary': 'org.apache.hadoop.hive.ql.io.parquet.serde.ParquetHiveSerDe'
        },
        'SkewedInfo': {
            'SkewedColumnNames': [],
            'SkewedColumnValueLocationMaps': {},
            'SkewedColumnValues': []
        },
        'SortColumns': [],
        'StoredAsSubDirectories': False
    },
    'TableType': 'EXTERNAL_TABLE',
}


PARTITION_INPUT = {
    # 'DatabaseName': 'dbname',
    'StorageDescriptor': {
        'BucketColumns': [],
        'Columns': [],
        'Compressed': False,
        'InputFormat': 'org.apache.hadoop.hive.ql.io.parquet.MapredParquetInputFormat',
        'Location': 's3://.../partition=value',
        'NumberOfBuckets': -1,
        'OutputFormat': 'org.apache.hadoop.hive.ql.io.parquet.MapredParquetOutputFormat',
        'Parameters': {},
        'SerdeInfo': {
            'Parameters': {'path': 's3://...', 'serialization.format': '1'},
            'SerializationLibrary': 'org.apache.hadoop.hive.ql.io.parquet.serde.ParquetHiveSerDe'},
        'SkewedInfo': {'SkewedColumnNames': [],
                       'SkewedColumnValueLocationMaps': {},
                       'SkewedColumnValues': []},
        'SortColumns': [],
        'StoredAsSubDirectories': False,
    },
    # 'TableName': 'source_table',
    # 'Values': ['2018-06-26'],
}



from __future__ import unicode_literals

import boto3
from botocore.exceptions import ClientError
from datetime import datetime, timedelta
import pytz
import sure   # noqa

from moto import mock_cloudwatch


@mock_cloudwatch
def test_put_list_dashboard():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'

    client.put_dashboard(DashboardName='test1', DashboardBody=widget)
    resp = client.list_dashboards()

    len(resp['DashboardEntries']).should.equal(1)


@mock_cloudwatch
def test_put_list_prefix_nomatch_dashboard():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'

    client.put_dashboard(DashboardName='test1', DashboardBody=widget)
    resp = client.list_dashboards(DashboardNamePrefix='nomatch')

    len(resp['DashboardEntries']).should.equal(0)


@mock_cloudwatch
def test_delete_dashboard():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'

    client.put_dashboard(DashboardName='test1', DashboardBody=widget)
    client.put_dashboard(DashboardName='test2', DashboardBody=widget)
    client.put_dashboard(DashboardName='test3', DashboardBody=widget)
    client.delete_dashboards(DashboardNames=['test2', 'test1'])

    resp = client.list_dashboards(DashboardNamePrefix='test3')
    len(resp['DashboardEntries']).should.equal(1)


@mock_cloudwatch
def test_delete_dashboard_fail():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'

    client.put_dashboard(DashboardName='test1', DashboardBody=widget)
    client.put_dashboard(DashboardName='test2', DashboardBody=widget)
    client.put_dashboard(DashboardName='test3', DashboardBody=widget)
    # Doesnt delete anything if all dashboards to be deleted do not exist
    try:
        client.delete_dashboards(DashboardNames=['test2', 'test1', 'test_no_match'])
    except ClientError as err:
        err.response['Error']['Code'].should.equal('ResourceNotFound')
    else:
        raise RuntimeError('Should of raised error')

    resp = client.list_dashboards()
    len(resp['DashboardEntries']).should.equal(3)


@mock_cloudwatch
def test_get_dashboard():
    client = boto3.client('cloudwatch', region_name='eu-central-1')
    widget = '{"widgets": [{"type": "text", "x": 0, "y": 7, "width": 3, "height": 3, "properties": {"markdown": "Hello world"}}]}'
    client.put_dashboard(DashboardName='test1', DashboardBody=widget)

    resp = client.get_dashboard(DashboardName='test1')
    resp.should.contain('DashboardArn')
    resp.should.contain('DashboardBody')
    resp['DashboardName'].should.equal('test1')


@mock_cloudwatch
def test_get_dashboard_fail():
    client = boto3.client('cloudwatch', region_name='eu-central-1')

    try:
        client.get_dashboard(DashboardName='test1')
    except ClientError as err:
        err.response['Error']['Code'].should.equal('ResourceNotFound')
    else:
        raise RuntimeError('Should of raised error')


@mock_cloudwatch
def test_alarm_state():
    client = boto3.client('cloudwatch', region_name='eu-central-1')

    client.put_metric_alarm(
        AlarmName='testalarm1',
        MetricName='cpu',
        Namespace='blah',
        Period=10,
        EvaluationPeriods=5,
        Statistic='Average',
        Threshold=2,
        ComparisonOperator='GreaterThanThreshold',
    )
    client.put_metric_alarm(
        AlarmName='testalarm2',
        MetricName='cpu',
        Namespace='blah',
        Period=10,
        EvaluationPeriods=5,
        Statistic='Average',
        Threshold=2,
        ComparisonOperator='GreaterThanThreshold',
    )

    # This is tested implicitly as if it doesnt work the rest will die
    client.set_alarm_state(
        AlarmName='testalarm1',
        StateValue='ALARM',
        StateReason='testreason',
        StateReasonData='{"some": "json_data"}'
    )

    resp = client.describe_alarms(
        StateValue='ALARM'
    )
    len(resp['MetricAlarms']).should.equal(1)
    resp['MetricAlarms'][0]['AlarmName'].should.equal('testalarm1')
    resp['MetricAlarms'][0]['StateValue'].should.equal('ALARM')

    resp = client.describe_alarms(
        StateValue='OK'
    )
    len(resp['MetricAlarms']).should.equal(1)
    resp['MetricAlarms'][0]['AlarmName'].should.equal('testalarm2')
    resp['MetricAlarms'][0]['StateValue'].should.equal('OK')

    # Just for sanity
    resp = client.describe_alarms()
    len(resp['MetricAlarms']).should.equal(2)


@mock_cloudwatch
def test_put_metric_data_no_dimensions():
    conn = boto3.client('cloudwatch', region_name='us-east-1')

    conn.put_metric_data(
        Namespace='tester',
        MetricData=[
            dict(
                MetricName='metric',
                Value=1.5,
            )
        ]
    )

    metrics = conn.list_metrics()['Metrics']
    metrics.should.have.length_of(1)
    metric = metrics[0]
    metric['Namespace'].should.equal('tester')
    metric['MetricName'].should.equal('metric')



@mock_cloudwatch
def test_put_metric_data_with_statistics():
    conn = boto3.client('cloudwatch', region_name='us-east-1')

    conn.put_metric_data(
        Namespace='tester',
        MetricData=[
            dict(
                MetricName='statmetric',
                Timestamp=datetime(2015, 1, 1),
                # no Value to test  https://github.com/spulec/moto/issues/1615
                StatisticValues=dict(
                    SampleCount=123.0,
                    Sum=123.0,
                    Minimum=123.0,
                    Maximum=123.0
                ),
                Unit='Milliseconds',
                StorageResolution=123
            )
        ]
    )

    metrics = conn.list_metrics()['Metrics']
    metrics.should.have.length_of(1)
    metric = metrics[0]
    metric['Namespace'].should.equal('tester')
    metric['MetricName'].should.equal('statmetric')
    # TODO: test statistics - https://github.com/spulec/moto/issues/1615

@mock_cloudwatch
def test_get_metric_statistics():
    conn = boto3.client('cloudwatch', region_name='us-east-1')
    utc_now = datetime.now(tz=pytz.utc)

    conn.put_metric_data(
        Namespace='tester',
        MetricData=[
            dict(
                MetricName='metric',
                Value=1.5,
                Timestamp=utc_now
            )
        ]
    )

    stats = conn.get_metric_statistics(
        Namespace='tester',
        MetricName='metric',
        StartTime=utc_now - timedelta(seconds=60),
        EndTime=utc_now + timedelta(seconds=60),
        Period=60,
        Statistics=['SampleCount', 'Sum']
    )

    stats['Datapoints'].should.have.length_of(1)
    datapoint = stats['Datapoints'][0]
    datapoint['SampleCount'].should.equal(1.0)
    datapoint['Sum'].should.equal(1.5)
